@@ -19,9 +19,9 @@ from http.server import HTTPServer, BaseHTTPRequestHandler
 from jinja2 import __version__ as jinja2_version
 import jinjafx, os, io, sys, socket, signal, threading, yaml, json, base64, time, datetime
 import re, argparse, zipfile, hashlib, traceback, glob, hmac, uuid, struct, binascii, gzip, requests
-import cmarkgfm, emoji
+import cmarkgfm, emoji, func_timeout
 
-__version__ = '22.7.6'
+__version__ = '22.8.0'
 
 lock = threading.RLock()
 base = os.path.abspath(os.path.dirname(__file__))
@@ -35,6 +35,7 @@ verbose = False
 rtable = {}
 rl_rate = 0
 rl_limit = 0
+timelimit = 0
 
 class JinjaFxServer(HTTPServer):
   def handle_error(self, request, client_address):
@@ -90,7 +91,10 @@ class JinjaFxRequest(BaseHTTPRequestHandler):
             log('[' + src + '] [\033[1;' + ansi + 'm' + str(args[1]) + '\033[0m]' + ' \033[1;' + ansi + 'm' + str(args[2]) + '\033[0m')
         
           elif self.command == 'POST':
-            log('[' + src + '] [\033[1;' + ansi + 'm' + str(args[1]) + '\033[0m]' + ' \033[1;33m' + self.command + '\033[0m ' + path + ctype + ' [' + self.format_bytes(self.length) + ']')
+            if self.elapsed is not None:
+              log('[' + src + '] [\033[1;' + ansi + 'm' + str(args[1]) + '\033[0m]' + ' \033[1;33m' + self.command + '\033[0m ' + path + ctype + ' [' + self.format_bytes(self.length) + '] in ' + str(self.elapsed) + 'ms')
+            else:
+              log('[' + src + '] [\033[1;' + ansi + 'm' + str(args[1]) + '\033[0m]' + ' \033[1;33m' + self.command + '\033[0m ' + path + ctype + ' [' + self.format_bytes(self.length) + ']')
 
           elif self.command != None:
             if (args[1] != '200' and args[1] != '304') or (not path.endswith('.js') and not path.endswith('.css') and not path.endswith('.png')) or verbose:
@@ -281,6 +285,7 @@ class JinjaFxRequest(BaseHTTPRequestHandler):
   def do_POST(self):
     self.critical = False
     self.hide = False
+    self.elapsed = None
 
     uc = self.path.split('?', 1)
     params = { x[0]: x[1] for x in [x.split('=') for x in uc[1].split('&') ] } if len(uc) > 1 else { }
@@ -323,9 +328,17 @@ class JinjaFxRequest(BaseHTTPRequestHandler):
                 if y != None:
                   gvars.update(y)
   
-              st = round(time.time() * 1000)
-              outputs = jinjafx.JinjaFx().jinjafx(template.decode('utf-8'), data.decode('utf-8'), gvars, 'Output')
-              ocount = 0
+              try:
+                st = round(time.time() * 1000)
+                ocount = 0
+      
+                if timelimit > 0:
+                  outputs = func_timeout.func_timeout(timelimit, jinjafx.JinjaFx().jinjafx, args=(template.decode('utf-8'), data.decode('utf-8'), gvars, 'Output', [], True))
+                else:
+                  outputs = jinjafx.JinjaFx().jinjafx(template.decode('utf-8'), data.decode('utf-8'), gvars, 'Output', [], True)
+                            
+              except func_timeout.FunctionTimedOut:
+                raise Exception("execution time limit of " + str(timelimit) + "s exceeded")
   
               jsr = {
                 'status': 'ok',
@@ -333,6 +346,8 @@ class JinjaFxRequest(BaseHTTPRequestHandler):
                 'outputs': {}
               }
 
+              self.elapsed = jsr['elapsed']
+
               def html_escape(text):
                 text = text.replace("'", "&apos;")
                 text = text.replace('"', "&quot;")
@@ -342,7 +357,7 @@ class JinjaFxRequest(BaseHTTPRequestHandler):
                 (oname, oformat) = o.rsplit(':', 1) if ':' in o else (o, 'text')
                 output = '\n'.join(outputs[o]) + '\n'
                 if len(output.strip()) > 0:
-                  if oformat == 'markdown':
+                  if oformat == 'markdown' or oformat == 'md':
                     o = oname + ':html'
                     options = (cmarkgfm.cmark.Options.CMARK_OPT_GITHUB_PRE_LANG | cmarkgfm.cmark.Options.CMARK_OPT_SMART | cmarkgfm.cmark.Options.CMARK_OPT_UNSAFE)
                     output = cmarkgfm.github_flavored_markdown_to_html(html_escape(output), options).replace('&amp;amp;', '&amp;').replace('&amp;', '&')
@@ -679,6 +694,7 @@ def main(rflag=[0]):
   global repository
   global rl_rate
   global rl_limit
+  global timelimit
   global verbose
 
   try:
@@ -693,6 +709,7 @@ def main(rflag=[0]):
     group_ex.add_argument('-r', metavar='<repository>', type=w_directory)
     group_ex.add_argument('-s3', metavar='<aws s3 url>', type=str)
     parser.add_argument('-rl', metavar='<rate/limit>', type=rlimit)
+    parser.add_argument('-tl', metavar='<time limit>', type=int, default=0)
     parser.add_argument('-v', action='store_true', default=False)
     args = parser.parse_args()
     verbose = args.v
@@ -719,6 +736,8 @@ def main(rflag=[0]):
 
       rl_rate = int(args.rl[0])
 
+    timelimit = args.tl
+
     def signal_handler(*args):
       rflag[0] = 2
 
